/* Weather_Inside_SD.ino
 * 2020-08-29 prefontDon@gmail.com
 * 
 * Weather stats are continually being measured via a 5v BME280.
 * Every 60 seconds the program will write the current stats to the SD card (THBP.txt) and play a short melody.
 * Whenever the PIR motion detector senses movement, it will write the current stats to the OLED display
 * and enable the RGB LED for 10s.
 * 
 * OLED display example follows:
 * 
 *   WEATHER INSIDE
 *  T:  22c  73f          // Temperature in C and F.
 *  H:  55%               // Humidity.
 *  DP: 9c  48f           // Dew Point.
 *  HX: 23c 73f           // Humidex.
 *  BP: 100.1kPa          // Barometric Pressure in kilo Pascals.
 *  20/08/29 15:04 !      // Date Time. ! means SD card write.
 */
   
/* Temp/Humidity/BP VARIABLES LIST
 * float Tc = bme.readTemperature();           // Temp in C.                  
 * float Tf = ((Tc * 1.8) + 32);               // Temp in F.
 * float H = bme.readHumidity();               // Humidity %.
 * float TDPc = ((100 - H) / 5);               // Dew Point temperature in C.
 * float TDPf = ((TDPc * 1.8) + 32);           // Dew Point temperature in F.
 * float TDPk = TDPc + 273.15;                 // Dew Point temperature in K. 
 * Calculate Humidex in C based on Dew Point and Humidity.
 *   float HXc = Tc + (0.5555 * (pow(6.11,(5417.7530 * ((1/273.16) - (1/TDPk))) - 10)));                                        
 * float HXf = ((HXc * 1.8) + 32);             // Humidex in F.
 * float BP = (bme.readPressure() / 1000.0F);  // Barometric pressure in kilo Pascals kPa (Pa / 1000).
 */

int LED_13 = 13;                   // Pin D13. Also used by SPI SCK so it may flash during SD card access.

#include <Wire.h>                  // For I2C.
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
Adafruit_BME280 bme;               // I2C
#define SEALEVELPRESSURE_HPA (1026.00)  // Google "Barometric Pressure YourCity" and place here.

#include <U8x8lib.h>               // Economical text-only OLED display driver!
// The complete U8x8 Contructor List is available here: https://github.com/olikraus/u8g2/wiki/u8x8setupcpp
// Use U8X8_PIN_NONE if the Reset pin is not available on your OLED display.
// There are two versions of the I2C 128x64 1.3" OLED. Uncomment the one you are using:
// U8X8_SH1106_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE);   // 4-pin I2C 1.3" 128x64 OLED with no Reset.
   U8X8_SSD1306_128X64_NONAME_HW_I2C u8x8(/* reset=*/ U8X8_PIN_NONE);  // 4-pin I2C 1.3" 128x64 OLED with no Reset.
/* NOTE: The Adafruit SSD1306 drivers and graphics library are not used. Instead, the generic U8X8 SH1106 drivers 
 *       are used. This means both of the cheap OLEDs can be used: the ones pinned as VCC|GND|SCL|SDA or 
 *       GND|VCC|SCL|SDA, both of which do not have a RESET pin. 
*/
#include <RTClib.h>                // Real time clock library, uses I2C.
RTC_DS1307 rtc;                    
//char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

// Include the SD library:
#include <SPI.h>                   // Hardware SPI library: D10:SS, D11:MOSI, D12:MISO, D13:SCK.
#include <SD.h>                    // SD library.
File statsFile;                    // SD device name to write stats to; filename will be THBP.txt.

unsigned long delayTime = 2500;
long oledOnTime = 5000;            // Milliseconds of OLED display on-time once PIR is triggered.
long SdWriteTime = 5000;           // Milliseconds (60s) between stats write to SD card.

unsigned long currentMillis = 0;    // Milliseconds since reboot.
unsigned long previousMillis = 0;   // Milliseconds since last recording.  
unsigned long oledMillis = 0;       // OLED on-time in ms.
unsigned long totalOledMillis = 0;  
unsigned long sdMillis = 0;         // 
unsigned long totalSdMillis = 0;    //
////////////////////////////////////////////////////////////
void setup() {
    Serial.begin(9600);
    while(!Serial);                  // Time to get serial monitor running.
    Serial.println(F("BME280 test"));
    unsigned status = bme.begin();   // Default BME280 sensor settings for I2C address, etc.
    // You can also pass in a Wire library object like &Wire2: "status = bme.begin(0x76, &Wire2);"
    if (!status) {
        Serial.println(F("Could not find a valid BME280 sensor, check wiring, address, sensor ID!"));
        Serial.print(F("SensorID was: 0x")); 
        Serial.println(bme.sensorID(),16);
        Serial.print(F("        ID of 0xFF probably means a bad address, a BMP 180 or BMP 085\n"));
        Serial.print(F("   ID of 0x56-0x58 represents a BMP 280,\n"));
        Serial.print(F("        ID of 0x60 represents a BME 280.\n"));
        Serial.print(F("        ID of 0x61 represents a BME 680.\n"));
        while (1) delay(10);
    } 
    Serial.println(F("-- Default Test --"));
    Serial.println();

  Serial.print(F("Initializing SD card..."));
  pinMode(10, OUTPUT);               // Note that even if it's not used as the CS pin, the hardware SS pin (10) 
                                     // must be left as an output or the SD library functions will not work. 
  if (!SD.begin(10)) {
    Serial.println(F(" SD initialization failed!\n"));
    return;  }
  Serial.println(F("SD initialization done.\n"));

  u8x8.begin();               // Start OLED: initDisplay, clearDisplay, setPowerSave.
  u8x8.setPowerSave(0);       // Disable OLED power save function. https://github.com/olikraus/u8g2/wiki/u8x8reference#setpowersave
  u8x8.clearDisplay();        // Clears entire display, not just one line.
  bme.begin();                // Start BME280.        
  rtc.begin();                // Start RTC. 
//The following line sets the RTC to the date & time this sketch was compiled. Run it once to set the time automatically, 
//then comment it out and run it again; the battery-backed time will be used.
  rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));  
  outOLED();  
}
////////////////////////////////////////////////////////////
void loop() { 
  printValues();
  outOLED();
  delay(2500);
  u8x8.clearDisplay();        // Clears entire display, not just one line.
  currentMillis = millis();
  totalOledMillis = totalOledMillis + (currentMillis - previousMillis); 
  totalSdMillis = totalSdMillis + (currentMillis - previousMillis); 

  if(totalSdMillis >=  SdWriteTime) {
    totalSdMillis = 0;  
    outSD();                            // Write stats to SD and play tone if time between writes exceeds threshold.
 // playTone_1();  
  }  
}
////////////////////////////////////////////////////////////
void printValues() {
    Serial.print(F("Temperature = "));
    Serial.print(bme.readTemperature(),0);                  // Default temp is Celsius.
    Serial.print(F("C, "));
    Serial.print((bme.readTemperature() * 1.8) + 32,0);     // Converted to Fahrenheit.
    Serial.println(F("F"));

    Serial.print(F("Pressure = "));
    Serial.print(bme.readPressure() / 1000.0F,1 );           // Divide by 1000 to get kilo Pascals.
    Serial.print(F(" kPa, "));   
    Serial.print((bme.readPressure() / 3386.39) * 25.4,0);  // 1 inch mercury = 3386.39Pa. 1 inch = 25.4mm.
    Serial.println(F(" mmHg"));

    Serial.print(F("Humidity = "));
    Serial.print(bme.readHumidity(),0);
    Serial.println(F(" %"));
    
    Serial.print(F("Approx. Altitude = "));
    Serial.print(bme.readAltitude(SEALEVELPRESSURE_HPA),0);
    Serial.println(F(" m"));
    Serial.println();
}
////////////////////////////////////////////////////////////
void outOLED() {
  DateTime now = rtc.now();         // Must be in each function's scope.
  u8x8.setFont(u8x8_font_chroma48medium8_r); // Font chosen.
  u8x8.drawString(0,0," WEATHER INSIDE");    // Display the program title.
  
  u8x8.drawString(0,2,"T: ");       // Display Temperature in C (default) and F.
  u8x8.setCursor(4,2);
  float Tc = bme.readTemperature(); // Temp in C.
  u8x8.print(Tc,0); 
  u8x8.drawString(6,2,"c");                             
  u8x8.setCursor(8,2);
  float Tf = ((Tc * 1.8) + 32);     // Temp in F.
  u8x8.print(Tf,0);     
  u8x8.drawString(10,2,"f");  

  u8x8.drawString(0,3,"H: ");       // Display Humidity in %.
  u8x8.setCursor(4,3);
  float H = bme.readHumidity();     // Humidity %.
  u8x8.print(H,0);
  u8x8.drawString(6,3,"%");

  u8x8.drawString(0,4,"DP:");       // Display Dew Point in C and F.
  u8x8.setCursor(4,4);
  float TDPc = ((100 - H) / 5);     // Dew Point temperature in C.
  u8x8.print(TDPc,0);
  u8x8.drawString(6,4,"c"); 
  u8x8.setCursor(8,4);  
  float TDPf = ((TDPc * 1.8) + 32); // Dew Point temperature in F.
  u8x8.print(TDPf,0);
  u8x8.drawString(10,4,"f");
  
  u8x8.drawString(0,5,"HX: ");      // Display Humidex in C and F using formula at bottom.
  u8x8.setCursor(4,5);
  float TDPk = TDPc + 273.15;       // Dew Point temperature in K. 
  float HXc = Tc + (0.5555 * (pow(6.11,(5417.7530 * ((1/273.16) - (1/TDPk))) - 10)));  // Calculate Humidex in C based on Dew Point and Humidity.
  u8x8.print(HXc,0);
  u8x8.drawString(6,5,"c");  
  u8x8.setCursor(8,5);  
  float HXf = ((HXc * 1.8) + 32);   // Humidex in F.
  u8x8.print(HXf,0);      
  u8x8.drawString(10,5,"f"); 

  u8x8.drawString(0,6,"BP:");       // Display Barometric Pressure in kilo Pascals (kPa)
  u8x8.setCursor(4,6); 
  float BP = (bme.readPressure() / 1000.0F);  // Barometric pressure in kilo Pascals kPa (Pa / 1000).
  u8x8.print(BP,1);
  u8x8.drawString(10,6,"kPa");  
//float BPmm = (BP * 7.50062);      // Barometric pressure in millimeters mercury (mmHg).
//u8x8.print(BPmm,1);
//u8x8.drawString(10,6,"mmHg");
//float BPin = (BP * 0.2953);       // Barometric pressure in inches mercury (inHg).
//u8x8.print(BPin,1);
//u8x8.drawString(10,6,"inHg"); 

  u8x8.setCursor(0,7);              // YY/MM/DD HH:MM
  u8x8.print(now.year(),DEC);
  u8x8.drawString(2,7,"/");
  u8x8.setCursor(3,7);
  u8x8.print(now.month(),DEC);
  u8x8.drawString(5,7,"/");
  u8x8.setCursor(6,7);
  u8x8.print(now.day(), DEC);    
  u8x8.drawString(8,7," ");
  u8x8.setCursor(9,7);
  u8x8.print(now.hour(), DEC);    
  u8x8.drawString(11,7,":");
  u8x8.setCursor(12,7);
  u8x8.print(now.minute(), DEC);
  u8x8.drawString(14,7," ");
//u8x8.drawString(14,7,":");
//u8x8.print(now.second(), DEC);
//u8x8.setCursor(0,8);
//u8x8l.print(daysOfTheWeek[now.dayOfTheWeek()]);  
}
///////////////////////////////////////////////////////////////////
void mills(void) {                  // Keeps track of both OLED and SD times.
  Serial.print(F("currentMillis("));
  Serial.print(currentMillis);
  Serial.print(F(" ms)"));
  Serial.print(F(" - previousMillis("));
  Serial.print(previousMillis);
  Serial.print(F(" ms)"));
  Serial.print(F(" = "));
  Serial.print(currentMillis - previousMillis);
  Serial.print(F(" ms\t"));
  
  Serial.print(F("totalOledMillis = "));
  Serial.print(F(" ms)"));
  Serial.print(totalOledMillis);
  Serial.print(F(" ms\t"));

  Serial.print(F("totalSdMillis = "));
  Serial.print(F(" ms)"));
  Serial.print(totalSdMillis);
  Serial.print(F(" ms\t"));

  float BP = (bme.readPressure() / 1000.0F);  // Barometric pressure in kilo Pascals kPa (Pa / 1000).
  float BPmm = (BP * 7.50062);      // Barometric pressure in millimeters mercury (mmHg).
  float BPin = (BP * 0.2953);       // Barometric pressure in inches mercury (inHg).
  Serial.print(F("BP = "));
  Serial.print(BP,1);
  Serial.print(F("kPa "));
  Serial.print(BPmm,1);
  Serial.print(F("mmHg "));
  Serial.print(BPin,2);
  Serial.print(F("inHg\t"));
      
  Serial.print(F("pirState = "));
//  Serial.println(pirState);
}  
///////////////////////////////////////////////////////////////////
void outSD() {
   DateTime now = rtc.now();         // Must be in each function's scope.
// Open the file. Note that only one file can be open at a time so you have to close the current one before opening another.
   statsFile = SD.open("THBP.txt", FILE_WRITE);      // SPI. 
   if (statsFile) {                                  // If the file opened okay, write to it and to Serial Monitor.
     Serial.println(F("\nWriting to THBP.txt...\n"));
   } else {
     Serial.println(F("\nError opening THBP.txt\n")); // Print an error to the Serial Monitor if the file didn't open.
  }      
  statsFile.print(now.year(),DEC);   // Write date and time to SD: YY/MM/DD HH:MM.
  statsFile.print(F("/"));
  statsFile.print(now.month(),DEC);
  statsFile.print(F("/"));
  statsFile.print(now.day(), DEC); 
  statsFile.print(F(" "));  
  statsFile.print(now.hour(), DEC);  
  statsFile.print(F(":"));    
  statsFile.print(now.minute(), DEC);
  statsFile.print(F(":"));
  statsFile.print(now.second(), DEC);
  
  statsFile.print(F("\t\tT:\t"));   // T:xxc xxf
  float Tc = bme.readTemperature(); // Temp in C.
  statsFile.print(Tc,0);            
  statsFile.print(F("c\t")); 
  float Tf = ((Tc * 1.8) + 32);     // Temp in F.
  statsFile.print(Tf,0);
  statsFile.print(F("f"));  

  statsFile.print(F("\t\tH:\t"));   // H:%
  float H = bme.readHumidity();     // Humidity %.
  statsFile.print(H,0);
  statsFile.print(F("%"));  
  statsFile.print("\t\tDP:\t");     // DP:xxc xxf
  float TDPc = ((100 - H) / 5);     // Dew Point temperature in C.
  statsFile.print(TDPc,0);
  statsFile.print(F("c\t"));
  float TDPf = ((TDPc * 1.8) + 32); // Dew Point temperature in F.
  statsFile.print(TDPf,0);
  statsFile.print(F("f"));

  statsFile.print(F("\t\tHX:\t"));  // HX:xxc xxf
  float TDPk = TDPc + 273.15;       // Dew Point temperature in K. 
  float HXc = Tc + (0.5555 * (pow(6.11,(5417.7530 * ((1/273.16) - (1/TDPk))) - 10)));  // Calculate Humidex in C based on Dew Point and Humidity.
  statsFile.print(HXc,0);
  statsFile.print(F("c\t")); 
  float HXf = ((HXc * 1.8) + 32);   // Humidex in F.
  statsFile.print(HXf,0);  
  statsFile.print(F("f")); 

  statsFile.print(F("\t\tBP:\t"));  // HX:xxc xxf
  float BP = (bme.readPressure() / 1000.0F);  // Barometric pressure in kilo Pascals kPa (Pa / 1000).
  float BPmm = (BP * 7.50062);      // Barometric pressure in millimeters mercury (mmHg).
  float BPin = (BP * 0.2953);       // Barometric pressure in inches mercury (inHg).
  statsFile.print(BP,1);
  statsFile.print(F(" kPa\t"));   // kilo Pascals.
  statsFile.print(BPmm,1);
  statsFile.print(F(" mmHg\t"));  // millimeters Mercury.
  statsFile.print(BPin,1);  
  statsFile.println(F(" inHg"));  // inches Mercury.
  statsFile.close();                  
}
///////////////////////////////////////////////////////////////////

// humidex = air temperature + (0.5555)*(e - 10.0) where e = 6.11 * exp [5417.7530 * ((1/273.16) - (1/dewpoint in kelvins))]
